"""
OMOP CDM vocabulary tables.

CDM 6.0.0 check constraints are included.
"""

import datetime
import decimal
from typing import Optional

from sqlalchemy import CheckConstraint, Date, DateTime, ForeignKey, Integer, Numeric, String
from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy.orm import Mapped, mapped_column, relationship

from omop_cdm.constants import (
    CDM_SCHEMA,
    FK_CONCEPT_CLASS_ID,
    FK_CONCEPT_ID,
    FK_DOMAIN_ID,
    FK_VOCABULARY_ID,
    VOCAB_SCHEMA,
)
from omop_cdm.util import get_current_time_utc


class BaseConceptCdm600:
    __tablename__ = "concept"
    __table_args__ = (
        CheckConstraint("(COALESCE(invalid_reason,'D') in ('D','U'))", name="chk_c_invalid_reason"),
        CheckConstraint("(COALESCE(standard_concept,'C') in ('C','S'))", name="chk_c_standard_concept"),
        CheckConstraint("(concept_code <> '')", name="chk_c_concept_code"),
        CheckConstraint("(concept_name <> '')", name="chk_c_concept_name"),
        {"schema": VOCAB_SCHEMA},
    )

    concept_id: Mapped[int] = mapped_column(Integer, primary_key=True, sort_order=100)
    concept_name: Mapped[str] = mapped_column(String(255), sort_order=200)
    domain_id: Mapped[str] = mapped_column(ForeignKey(FK_DOMAIN_ID), index=True, sort_order=300)
    vocabulary_id: Mapped[str] = mapped_column(ForeignKey(FK_VOCABULARY_ID), index=True, sort_order=400)
    concept_class_id: Mapped[str] = mapped_column(ForeignKey(FK_CONCEPT_CLASS_ID), index=True, sort_order=500)
    standard_concept: Mapped[Optional[str]] = mapped_column(String(1), sort_order=600)
    concept_code: Mapped[str] = mapped_column(String(50), index=True, sort_order=700)
    valid_start_date: Mapped[datetime.date] = mapped_column(Date, sort_order=800)
    valid_end_date: Mapped[datetime.date] = mapped_column(Date, sort_order=900)
    invalid_reason: Mapped[Optional[str]] = mapped_column(String(1), sort_order=1000)

    @declared_attr
    def concept_class(cls) -> Mapped["ConceptClass"]:
        return relationship("ConceptClass", foreign_keys="Concept.concept_class_id")

    @declared_attr
    def domain(cls) -> Mapped["Domain"]:
        return relationship("Domain", foreign_keys="Concept.domain_id")

    @declared_attr
    def vocabulary(cls) -> Mapped["Vocabulary"]:
        return relationship("Vocabulary", foreign_keys="Concept.vocabulary_id")


class BaseConceptAncestorCdm600:
    __tablename__ = "concept_ancestor"
    __table_args__ = {"schema": VOCAB_SCHEMA}

    ancestor_concept_id: Mapped[int] = mapped_column(
        ForeignKey(FK_CONCEPT_ID), primary_key=True, index=True, sort_order=100
    )
    descendant_concept_id: Mapped[int] = mapped_column(
        ForeignKey(FK_CONCEPT_ID), primary_key=True, index=True, sort_order=200
    )
    min_levels_of_separation: Mapped[int] = mapped_column(Integer, sort_order=300)
    max_levels_of_separation: Mapped[int] = mapped_column(Integer, sort_order=400)

    @declared_attr
    def ancestor_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="ConceptAncestor.ancestor_concept_id")

    @declared_attr
    def descendant_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="ConceptAncestor.descendant_concept_id")


class BaseConceptClassCdm600:
    __tablename__ = "concept_class"
    __table_args__ = {"schema": VOCAB_SCHEMA}

    concept_class_id: Mapped[str] = mapped_column(String(20), primary_key=True, sort_order=100)
    concept_class_name: Mapped[str] = mapped_column(String(255), sort_order=200)
    concept_class_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=300)

    @declared_attr
    def concept_class_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="ConceptClass.concept_class_concept_id")


class BaseConceptRelationshipCdm600:
    __tablename__ = "concept_relationship"
    __table_args__ = (
        CheckConstraint("(COALESCE(invalid_reason,'D')='D')", name="chk_cr_invalid_reason"),
        {"schema": VOCAB_SCHEMA},
    )

    concept_id_1: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), primary_key=True, index=True, sort_order=100)
    concept_id_2: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), primary_key=True, index=True, sort_order=200)
    relationship_id: Mapped[str] = mapped_column(
        ForeignKey(f"{VOCAB_SCHEMA}.relationship.relationship_id"), primary_key=True, index=True, sort_order=300
    )
    valid_start_date: Mapped[datetime.date] = mapped_column(Date, sort_order=400)
    valid_end_date: Mapped[datetime.date] = mapped_column(Date, sort_order=500)
    invalid_reason: Mapped[Optional[str]] = mapped_column(String(1), sort_order=600)

    @declared_attr
    def concept_1(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="ConceptRelationship.concept_id_1")

    @declared_attr
    def concept_2(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="ConceptRelationship.concept_id_2")

    @declared_attr
    def relationship(cls) -> Mapped["Relationship"]:
        return relationship("Relationship", foreign_keys="ConceptRelationship.relationship_id")


class BaseConceptSynonymCdm600:
    __tablename__ = "concept_synonym"
    __table_args__ = (
        # UniqueConstraint uq_concept_synonym is not added, because
        # there is already a PK on the same columns
        CheckConstraint("(concept_synonym_name <> '')", name="chk_csyn_concept_synonym_name"),
        {"schema": VOCAB_SCHEMA},
    )

    concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), primary_key=True, index=True, sort_order=100)
    concept_synonym_name: Mapped[str] = mapped_column(String(1000), primary_key=True, sort_order=200)
    language_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), primary_key=True, sort_order=300)

    @declared_attr
    def concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="ConceptSynonym.concept_id")

    @declared_attr
    def language_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="ConceptSynonym.language_concept_id")


class BaseDomainCdm600:
    __tablename__ = "domain"
    __table_args__ = {"schema": VOCAB_SCHEMA}

    domain_id: Mapped[str] = mapped_column(String(20), primary_key=True, sort_order=100)
    domain_name: Mapped[str] = mapped_column(String(255), sort_order=200)
    domain_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=300)

    @declared_attr
    def domain_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="Domain.domain_concept_id")


class BaseDrugStrengthCdm600:
    __tablename__ = "drug_strength"
    __table_args__ = {"schema": VOCAB_SCHEMA}

    drug_concept_id: Mapped[int] = mapped_column(
        ForeignKey(FK_CONCEPT_ID), primary_key=True, index=True, sort_order=100
    )
    ingredient_concept_id: Mapped[int] = mapped_column(
        ForeignKey(FK_CONCEPT_ID), primary_key=True, index=True, sort_order=200
    )
    amount_value: Mapped[Optional[decimal.Decimal]] = mapped_column(Numeric, sort_order=300)
    amount_unit_concept_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=400)
    numerator_value: Mapped[Optional[decimal.Decimal]] = mapped_column(Numeric, sort_order=500)
    numerator_unit_concept_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=600)
    denominator_value: Mapped[Optional[decimal.Decimal]] = mapped_column(Numeric, sort_order=700)
    denominator_unit_concept_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=800)
    box_size: Mapped[Optional[int]] = mapped_column(Integer, sort_order=900)
    valid_start_date: Mapped[datetime.date] = mapped_column(Date, sort_order=1000)
    valid_end_date: Mapped[datetime.date] = mapped_column(Date, sort_order=1100)
    invalid_reason: Mapped[Optional[str]] = mapped_column(String(1), sort_order=1200)

    @declared_attr
    def amount_unit_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="DrugStrength.amount_unit_concept_id")

    @declared_attr
    def denominator_unit_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="DrugStrength.denominator_unit_concept_id")

    @declared_attr
    def drug_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="DrugStrength.drug_concept_id")

    @declared_attr
    def ingredient_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="DrugStrength.ingredient_concept_id")

    @declared_attr
    def numerator_unit_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="DrugStrength.numerator_unit_concept_id")


class BaseRelationshipCdm600:
    __tablename__ = "relationship"
    __table_args__ = {"schema": VOCAB_SCHEMA}

    relationship_id: Mapped[str] = mapped_column(String(20), primary_key=True, sort_order=100)
    relationship_name: Mapped[str] = mapped_column(String(255), sort_order=200)
    is_hierarchical: Mapped[str] = mapped_column(String(1), sort_order=300)
    defines_ancestry: Mapped[str] = mapped_column(String(1), sort_order=400)
    reverse_relationship_id: Mapped[str] = mapped_column(
        ForeignKey(f"{VOCAB_SCHEMA}.relationship.relationship_id"), sort_order=500
    )
    relationship_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=600)

    @declared_attr
    def relationship_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="Relationship.relationship_concept_id")

    @declared_attr
    def reverse_relationship(cls) -> Mapped["Relationship"]:
        return relationship("Relationship", foreign_keys="Relationship.reverse_relationship_id")


class BaseVocabularyCdm600:
    __tablename__ = "vocabulary"
    __table_args__ = {"schema": VOCAB_SCHEMA}

    vocabulary_id: Mapped[str] = mapped_column(String(20), primary_key=True, sort_order=100)
    vocabulary_name: Mapped[str] = mapped_column(String(255), sort_order=200)
    vocabulary_reference: Mapped[str] = mapped_column(String(255), sort_order=300)
    vocabulary_version: Mapped[Optional[str]] = mapped_column(String(255), sort_order=400)
    vocabulary_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=500)

    @declared_attr
    def vocabulary_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="Vocabulary.vocabulary_concept_id")


class BaseSourceToConceptMapCdm600:
    __tablename__ = "source_to_concept_map"
    __table_args__ = {"schema": CDM_SCHEMA}

    source_code: Mapped[str] = mapped_column(String(1000), primary_key=True, index=True, sort_order=100)
    source_concept_id: Mapped[int] = mapped_column(Integer, sort_order=200)
    source_vocabulary_id: Mapped[str] = mapped_column(
        ForeignKey(FK_VOCABULARY_ID), primary_key=True, index=True, sort_order=300
    )
    source_code_description: Mapped[Optional[str]] = mapped_column(String(255), sort_order=400)
    target_concept_id: Mapped[int] = mapped_column(
        ForeignKey(FK_CONCEPT_ID), primary_key=True, index=True, sort_order=500
    )
    target_vocabulary_id: Mapped[str] = mapped_column(ForeignKey(FK_VOCABULARY_ID), index=True, sort_order=600)
    valid_start_date: Mapped[datetime.date] = mapped_column(Date, sort_order=700)
    valid_end_date: Mapped[datetime.date] = mapped_column(Date, primary_key=True, sort_order=800)
    invalid_reason: Mapped[Optional[str]] = mapped_column(String(1), sort_order=900)

    @declared_attr
    def source_vocabulary(cls) -> Mapped["Vocabulary"]:
        return relationship("Vocabulary", foreign_keys="SourceToConceptMap.source_vocabulary_id")

    @declared_attr
    def target_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="SourceToConceptMap.target_concept_id")

    @declared_attr
    def target_vocabulary(cls) -> Mapped["Vocabulary"]:
        return relationship("Vocabulary", foreign_keys="SourceToConceptMap.target_vocabulary_id")


class BaseSourceToConceptMapVersionCdm600:
    __tablename__ = "source_to_concept_map_version"
    __table_args__ = {"schema": CDM_SCHEMA}

    source_vocabulary_id: Mapped[str] = mapped_column(ForeignKey(FK_VOCABULARY_ID), primary_key=True, sort_order=100)
    stcm_version: Mapped[str] = mapped_column(String(255), sort_order=200)
    last_upload_date: Mapped[datetime.datetime] = mapped_column(DateTime, default=get_current_time_utc, sort_order=300)

    @declared_attr
    def source_vocabulary(cls) -> Mapped["Vocabulary"]:
        return relationship("Vocabulary", foreign_keys="SourceToConceptMapVersion.source_vocabulary_id")



"""
OMOP CDM Version 5.3.1.

Generated from OMOP CDM release 5.3.1 using sqlacodegen 3.0.0rc5.
DDL from https://github.com/OHDSI/CommonDataModel/tree/v5.3.1.



Deviations from standard model

Removed

attribute_definition
cohort_attribute


Added

stem_table
source_to_concept_map_version


Updated

source_to_concept_map
    source_code from VARCHAR(50) --> VARCHAR(1000)
    If using a separate vocab schema, STCM is created in the CDM schema

"""

from omop_cdm.dynamic.cdm531.clinical_data import (
    BaseConditionOccurrenceCdm531,
    BaseDeathCdm531,
    BaseDeviceExposureCdm531,
    BaseDrugExposureCdm531,
    BaseFactRelationshipCdm531,
    BaseMeasurementCdm531,
    BaseNoteCdm531,
    BaseNoteNlpCdm531,
    BaseObservationCdm531,
    BaseObservationPeriodCdm531,
    BasePersonCdm531,
    BaseProcedureOccurrenceCdm531,
    BaseSpecimenCdm531,
    BaseStemTableCdm531,
    BaseVisitDetailCdm531,
    BaseVisitOccurrenceCdm531,
)
from omop_cdm.dynamic.cdm531.derived_elements import (
    BaseConditionEraCdm531,
    BaseDoseEraCdm531,
    BaseDrugEraCdm531,
)
from omop_cdm.dynamic.cdm531.health_economics import (
    BaseCostCdm531,
    BasePayerPlanPeriodCdm531,
)
from omop_cdm.dynamic.cdm531.health_system_data import (
    BaseCareSiteCdm531,
    BaseLocationCdm531,
    BaseProviderCdm531,
)
from omop_cdm.dynamic.cdm531.metadata import (
    BaseCdmSourceCdm531,
    BaseMetadataCdm531,
)
from omop_cdm.dynamic.cdm531.vocabularies import (
    BaseConceptAncestorCdm531,
    BaseConceptCdm531,
    BaseConceptClassCdm531,
    BaseConceptRelationshipCdm531,
    BaseConceptSynonymCdm531,
    BaseDomainCdm531,
    BaseDrugStrengthCdm531,
    BaseRelationshipCdm531,
    BaseSourceToConceptMapCdm531,
    BaseSourceToConceptMapVersionCdm531,
    BaseVocabularyCdm531,
)



"""OMOP CDM 6.0.0 health economics tables."""

import datetime
import decimal
from typing import Optional

from sqlalchemy import BigInteger, Date, ForeignKey, Integer, Numeric, String
from sqlalchemy.orm import Mapped, declared_attr, mapped_column, relationship

from omop_cdm.constants import CDM_SCHEMA, FK_CONCEPT_ID, FK_PERSON_ID


class BaseCostCdm600:
    __tablename__ = "cost"
    __table_args__ = {"schema": CDM_SCHEMA}

    cost_id: Mapped[int] = mapped_column(BigInteger, primary_key=True, sort_order=100)
    person_id: Mapped[int] = mapped_column(ForeignKey(FK_PERSON_ID), index=True, sort_order=200)
    cost_event_id: Mapped[int] = mapped_column(BigInteger, sort_order=300)
    cost_event_field_concept_id: Mapped[int] = mapped_column(Integer, sort_order=400)
    cost_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=500)
    cost_type_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=600)
    currency_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=700)
    cost: Mapped[Optional[decimal.Decimal]] = mapped_column(Numeric, sort_order=800)
    incurred_date: Mapped[datetime.date] = mapped_column(Date, sort_order=900)
    billed_date: Mapped[Optional[datetime.date]] = mapped_column(Date, sort_order=1000)
    paid_date: Mapped[Optional[datetime.date]] = mapped_column(Date, sort_order=1100)
    revenue_code_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=1200)
    drg_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=1300)
    cost_source_value: Mapped[Optional[str]] = mapped_column(String(50), sort_order=1400)
    cost_source_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=1500)
    revenue_code_source_value: Mapped[Optional[str]] = mapped_column(String(50), sort_order=1600)
    drg_source_value: Mapped[Optional[str]] = mapped_column(String(3), sort_order=1700)
    payer_plan_period_id: Mapped[Optional[int]] = mapped_column(
        ForeignKey(f"{CDM_SCHEMA}.payer_plan_period.payer_plan_period_id"), sort_order=1800
    )

    @declared_attr
    def cost_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="Cost.cost_concept_id")

    @declared_attr
    def cost_source_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="Cost.cost_source_concept_id")

    @declared_attr
    def cost_type_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="Cost.cost_type_concept_id")

    @declared_attr
    def currency_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="Cost.currency_concept_id")

    @declared_attr
    def drg_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="Cost.drg_concept_id")

    @declared_attr
    def payer_plan_period(cls) -> Mapped["PayerPlanPeriod"]:
        return relationship("PayerPlanPeriod", foreign_keys="Cost.payer_plan_period_id")

    @declared_attr
    def person(cls) -> Mapped["Person"]:
        return relationship("Person", foreign_keys="Cost.person_id")

    @declared_attr
    def revenue_code_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="Cost.revenue_code_concept_id")


class BasePayerPlanPeriodCdm600:
    __tablename__ = "payer_plan_period"
    __table_args__ = {"schema": CDM_SCHEMA}

    payer_plan_period_id: Mapped[int] = mapped_column(BigInteger, primary_key=True, sort_order=100)
    person_id: Mapped[int] = mapped_column(ForeignKey(FK_PERSON_ID), index=True, sort_order=200)
    contract_person_id: Mapped[Optional[int]] = mapped_column(ForeignKey(FK_PERSON_ID), sort_order=300)
    payer_plan_period_start_date: Mapped[datetime.date] = mapped_column(Date, sort_order=400)
    payer_plan_period_end_date: Mapped[datetime.date] = mapped_column(Date, sort_order=500)
    payer_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=600)
    plan_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=700)
    contract_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=800)
    sponsor_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=900)
    stop_reason_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=1000)
    payer_source_value: Mapped[Optional[str]] = mapped_column(String(50), sort_order=1100)
    payer_source_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=1200)
    plan_source_value: Mapped[Optional[str]] = mapped_column(String(50), sort_order=1300)
    plan_source_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=1400)
    contract_source_value: Mapped[Optional[str]] = mapped_column(String(50), sort_order=1500)
    contract_source_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=1600)
    sponsor_source_value: Mapped[Optional[str]] = mapped_column(String(50), sort_order=1700)
    sponsor_source_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=1800)
    family_source_value: Mapped[Optional[str]] = mapped_column(String(50), sort_order=1900)
    stop_reason_source_value: Mapped[Optional[str]] = mapped_column(String(50), sort_order=2000)
    stop_reason_source_concept_id: Mapped[int] = mapped_column(ForeignKey(FK_CONCEPT_ID), sort_order=2100)

    @declared_attr
    def contract_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="PayerPlanPeriod.contract_concept_id")

    @declared_attr
    def contract_person(cls) -> Mapped["Person"]:
        return relationship("Person", foreign_keys="PayerPlanPeriod.contract_person_id")

    @declared_attr
    def contract_source_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="PayerPlanPeriod.contract_source_concept_id")

    @declared_attr
    def payer_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="PayerPlanPeriod.payer_concept_id")

    @declared_attr
    def payer_source_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="PayerPlanPeriod.payer_source_concept_id")

    @declared_attr
    def person(cls) -> Mapped["Person"]:
        return relationship("Person", foreign_keys="PayerPlanPeriod.person_id")

    @declared_attr
    def plan_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="PayerPlanPeriod.plan_concept_id")

    @declared_attr
    def plan_source_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="PayerPlanPeriod.plan_source_concept_id")

    @declared_attr
    def sponsor_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="PayerPlanPeriod.sponsor_concept_id")

    @declared_attr
    def sponsor_source_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="PayerPlanPeriod.sponsor_source_concept_id")

    @declared_attr
    def stop_reason_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="PayerPlanPeriod.stop_reason_concept_id")

    @declared_attr
    def stop_reason_source_concept(cls) -> Mapped["Concept"]:
        return relationship("Concept", foreign_keys="PayerPlanPeriod.stop_reason_source_concept_id")



"""
OMOP CDM Version 6.0.0.

Generated with python using sqlacodegen 3.0.0rc5, model from
https://github.com/OHDSI/CommonDataModel/tree/Dev/PostgreSQL,
commit 30d851a.



Deviations from standard model

Added

stem_table
source_to_concept_map_version


Updated

source_to_concept_map
    source_code from VARCHAR(50) --> VARCHAR(1000)
    If using a separate vocab schema, STCM is created in the CDM schema

"""

from omop_cdm.dynamic.cdm600.clinical_data import (
    BaseConditionOccurrenceCdm600,
    BaseDeviceExposureCdm600,
    BaseDrugExposureCdm600,
    BaseFactRelationshipCdm600,
    BaseMeasurementCdm600,
    BaseNoteCdm600,
    BaseNoteNlpCdm600,
    BaseObservationCdm600,
    BaseObservationPeriodCdm600,
    BasePersonCdm600,
    BaseProcedureOccurrenceCdm600,
    BaseSpecimenCdm600,
    BaseStemTableCdm600,
    BaseSurveyConductCdm600,
    BaseVisitDetailCdm600,
    BaseVisitOccurrenceCdm600,
)
from omop_cdm.dynamic.cdm600.derived_elements import (
    BaseConditionEraCdm600,
    BaseDoseEraCdm600,
    BaseDrugEraCdm600,
)
from omop_cdm.dynamic.cdm600.health_economics import (
    BaseCostCdm600,
    BasePayerPlanPeriodCdm600,
)
from omop_cdm.dynamic.cdm600.health_system_data import (
    BaseCareSiteCdm600,
    BaseLocationCdm600,
    BaseLocationHistoryCdm600,
    BaseProviderCdm600,
)
from omop_cdm.dynamic.cdm600.metadata import (
    BaseCdmSourceCdm600,
    BaseMetadataCdm600,
)
from omop_cdm.dynamic.cdm600.vocabularies import (
    BaseConceptAncestorCdm600,
    BaseConceptCdm600,
    BaseConceptClassCdm600,
    BaseConceptRelationshipCdm600,
    BaseConceptSynonymCdm600,
    BaseDomainCdm600,
    BaseDrugStrengthCdm600,
    BaseRelationshipCdm600,
    BaseSourceToConceptMapCdm600,
    BaseSourceToConceptMapVersionCdm600,
    BaseVocabularyCdm600,
)
